"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _events = _interopRequireDefault(require("events"));

var _ConsoleReporter = _interopRequireDefault(require("./ConsoleReporter"));

var _FileReporter = _interopRequireDefault(require("./FileReporter"));

var _SpinnerReporter = _interopRequireDefault(require("./SpinnerReporter"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Base Reporter distributing events to all defined reporters
 */
class Reporter extends _events.default {
  /**
   * @param {Object} config
   */
  constructor(config) {
    super();
    this._config = config;
    this._reporters = [];
  }
  /**
   * Initializes all reporters defined in config
   */


  init() {
    this._config.reporters.forEach(reporter => {
      if (reporter === 'console') {
        return this._initReporterFromClass(_ConsoleReporter.default);
      }

      if (reporter === 'file') {
        return this._initReporterFromClass(_FileReporter.default);
      }

      if (reporter === 'spinner') {
        return this._initReporterFromClass(_SpinnerReporter.default);
      }

      if (typeof reporter === 'string') {
        return this._initReporterFromString(reporter);
      }

      if (typeof reporter === 'function') {
        return this._initReporterFromClass(reporter);
      }

      throw Error('Failed to initalize custom reporter. Reporter must be defined via Class, or string.');
    });

    return this;
  }
  /**
   * Emits an event with data
   * and propagets it to all reporters
   * @param {string} eventName
   * @param {Object} eventData
   */


  emit(eventName, eventData) {
    this._reporters.forEach(reporter => {
      try {
        reporter.emit(eventName, eventData);
      } catch (e) {
        console.error(e);
      }
    });
  }
  /**
   * Initializes a reporter from string
   * @param {string} reporter
   */


  _initReporterFromString(reporter) {
    const reporterPath = `qape-reporter-${reporter}`;

    const Reporter = require(reporterPath).default;

    this._initReporterFromClass(Reporter);
  }
  /**
   * Initializes a reporter from a class constructor
   * @param {Class} Reporter
   */


  async _initReporterFromClass(Reporter) {
    let instance = new Reporter(this._config);

    if (typeof instance.init === 'function') {
      await instance.init();
    }

    this._reporters.push(instance);
  }

}

exports.default = Reporter;